unit MS_ClientForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, ModelSyncUtil, MS_Generator, ModelSyncThread;

type

TModelSyncClientForm = class(TForm)
  ERemoteIp: TEdit;
  ERemotePort: TEdit;
  Label5: TLabel;
  Label6: TLabel;
  BConnect: TButton;
  procedure FormCreate(Sender: TObject);
  procedure FormDestroy(Sender: TObject);
    procedure BConnectClick(Sender: TObject);
  private
    generator: TModelSyncGenerator;
    FThread: TMSClientThread;
    FMovingSyncThread: TMSMovingSyncThread;
  public
    { Public declarations }
  end;

var
  ModelSyncClientForm: TModelSyncClientForm;

implementation
uses
  ModelSyncPluginMain,
  PluginCore,
  WinSock;
{$R *.dfm}

procedure TModelSyncClientForm.BConnectClick(Sender: TObject);
  begin
    if BConnect.Caption = 'connect to server' then
      begin
        generator.Handle.RemoteHost:= Trim(ERemoteIp.Text);
        generator.Handle.RemotePort:= Trim(ERemotePort.Text);
        if (ERemoteIp.Text <> '') and (ERemotePort.Text <> '') then
           generator.Handle.Open;
        if not  generator.Handle.Connected then begin
          showmessage('failed, try again!');
          Exit;
        end;
        FMovingSyncThread:= TMSMovingSyncThread.Create(generator.Handle, true);
        FMovingSyncThread.Resume;
        FThread:= TMSClientThread.Create(generator.Handle, true);
        FThread.Resume;
        BConnect.Caption:= 'close connect';
      end
    else if BConnect.Caption = 'close connect' then
      begin
         generator.Handle.Close;
        if  generator.Handle.Active then begin
          showmessage('close failed, try again!');
          Exit;
        end;
        FThread.Terminate;
        FMovingSyncThread.Terminate;
        sleep(100);
        FMovingSyncThread.SocketHandle.Close;
        FreeAndNil(FMovingSyncThread);
        FThread.SocketHandle.Close;
        FreeAndNil(FThread);
        BConnect.Caption:= 'connect to server';
      end;
  end;

procedure TModelSyncClientForm.FormCreate(Sender: TObject);
  var
    method: TMethod;
  begin
    FThread:= nil;
    FMovingSyncThread:= nil;

    if not Assigned(generator) then
      generator:= TModelSyncGenerator.Create;

    FormMainModelClickFunc(method) := generator.MS_ModelOperationClick;
    ModelSyncPlugin.winRoadApplication.RegisterEventHandler( _plgFormMainModelClick, method);

    FormMainRoadClickFunc(method) := generator.MS_RoadOperationClick;
    ModelSyncPlugin.winRoadApplication.RegisterEventHandler( _plgFormMainRoadClick, method);

    FormMainTerrainClickProc(method) := generator.MS_TerrainOperationClick;
    ModelSyncPlugin.winRoadApplication.RegisterEventHandler(_plgFormMainTerrainClick,method);

    FormMainKeyUpDownProc(method) := generator.MS_Keyup;
    ModelSyncPlugin.winRoadApplication.RegisterEventHandler( _plgFormMainKeyUp, method);

    FormMainKeyUpDownProc(method) := generator.MS_KeyDown;
    ModelSyncPlugin.winRoadApplication.RegisterEventHandler( _plgFormMainKeyDown, method);
  end;

procedure TModelSyncClientForm.FormDestroy(Sender: TObject);
  var
    method: TMethod;
  begin
    if Assigned(generator) then begin
      FormMainModelClickFunc(method) := generator.MS_ModelOperationClick;
      ModelSyncPlugin.winRoadApplication.UnRegisterEventHandler( _plgFormMainModelClick, method);

      FormMainRoadClickFunc(method) := generator.MS_RoadOperationClick;
      ModelSyncPlugin.winRoadApplication.UnRegisterEventHandler( _plgFormMainRoadClick, method);

      FormMainTerrainClickProc(method) := generator.MS_TerrainOperationClick;
      ModelSyncPlugin.winRoadApplication.UnRegisterEventHandler(_plgFormMainTerrainClick,method);

      FormMainKeyUpDownProc(method) := generator.MS_Keyup;
      ModelSyncPlugin.winRoadApplication.UnRegisterEventHandler( _plgFormMainKeyUp, method);

      FormMainKeyUpDownProc(method) := generator.MS_KeyDown;
      ModelSyncPlugin.winRoadApplication.UnRegisterEventHandler( _plgFormMainKeyDown, method);

      FreeAndNil(generator);
    end;

    if Assigned(FThread) then begin
      FThread.Terminate;
      sleep(100);
      FThread.SocketHandle.Close;
      FreeAndNil(FThread);
    end;

    if Assigned(FMovingSyncThread) then begin
      FMovingSyncThread.Terminate;
      sleep(100);
      FMovingSyncThread.SocketHandle.Close;
      FreeAndNil(FMovingSyncThread);
    end;

  end;

end.

