unit MS_Operation;

interface

uses
  PluginCore,
  F8OpenGL,
  F8GLUtils;

const
  MODEL_SYNC_OPERATION                = 127;

  MODEL_SYNC_NONE                     = 0;
  MODEL_SYNC_CREATE_MODEL             = 2;
  MODEL_SYNC_DELETE_MODEL             = 4;
  MODEL_SYNC_ROTATE_MODEL             = 6;
  MODEL_SYNC_CREATE_OBSTRUCTION       = 8;
  MODEL_SYNC_DELETE_OBSTRUCTION       = 10;
  MODEL_SYNC_SYNC_MOVING_INSTANCE     = 12;
  MODEL_SYNC_CREATE_FIRE              = 14;
  MODEL_SYNC_COMMAND                  = 16;

type

// MS
// Աб
// =========FSerial룬Ĭ127
// =========FTypeĲ
// =========FFlagУֽ
// =========FIdĵǰ
// =========FValidǷΪЧ
TBaseOperation = class
  private
    FSerial: Byte;
    FType: Byte;
    FFlag: Byte;
    FId: Cardinal;
    FValid: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute; virtual; abstract;
    procedure Validate; virtual; abstract;

    property Id: Cardinal read FId write FId;
    property OperType: Byte read FType write FType;
    property Valid: Boolean read FValid write FValid;
    property Flag: Byte read FFlag write FFlag;
    property Serial: Byte read FSerial write FSerial;
  end;

// ģ͵MS
// Ա
// =========FPositionλϢ
// =========FModelIndexģϢ
TCreateModelOperation = class(TBaseOperation)
  private
    FPosition: GLPointType;
    FModelIndex: Integer;
  public
    constructor Create; overload;
    constructor Create(position: GLPointType; model: Integer); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property Position: GLPointType read FPosition write FPosition;
    property ModelIndex: Integer read FModelIndex write FModelIndex;
  end;

// ӻģ͵MS
// Ա
// =========FPositionλϢ
// =========FModelIndexģϢ
TCreateFireOperation = class(TBaseOperation)
  private
    FPosition: GLPointType;
    FModelIndex: Integer;
  public
    constructor Create; overload;
    constructor Create(position: GLPointType; model: Integer); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property Position: GLPointType read FPosition write FPosition;
    property ModelIndex: Integer read FModelIndex write FModelIndex;
  end;

// ӿMS
// Ա
// =========FTypeǼȡ
// =========FModelInstanceģ͵Ϣ
// =========FCommandIndexϢ
TCommandOperation = class(TBaseOperation)
  private
    FType: Byte;
    FModelInstance: Integer;
    FCommandIndex: Integer;
    FModelInstanceType: Byte;
  public
    constructor Create; overload;
    constructor Create(data, instanceType: Byte; modelInstance, commandIndex: Integer); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property CommandType: Byte read FType write FType;
    property ModelInstance: Integer read FModelInstance write FModelInstance;
    property CommandIndex: Integer read FCommandIndex write FCommandIndex;
    property ModelInstanceType: Byte read FModelInstanceType write FModelInstanceType;
  end;

// ɾģ͵MS
// Ա
// =========FModelInstanceɾϢ
// =========FModelInstanceTypeɾϢ
TDeleteModelOperation = class(TBaseOperation)
  private
    FModelInstance: Integer;
    FModelInstanceType: Byte;
  public
    constructor Create; overload;
    constructor Create(instance: Integer; instanceType: Byte); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property ModelInstance: Integer read FModelInstance write FModelInstance;
    property ModelInstanceType: Byte read FModelInstanceType write FModelInstanceType;
  end;

// תģ͵MS
// Ա
// =========FModelInstanceתϢ
// =========FModelInstanceTypeתϢ
// =========FYawAngleתǶϢ
TRotateModelOperation = class(TBaseOperation)
  private
    FModelInstance: Integer;
    FModelInstanceType: Byte;
    FYawAngle: Double;
  public
    constructor Create; overload;
    constructor Create(instance: Integer; instanceType: Byte; yawAngle: Double); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property ModelInstance: Integer read FModelInstance write FModelInstance;
    property ModelInstanceType: Byte read FModelInstanceType write FModelInstanceType;
    property YawAngle: Double read FYawAngle write FYawAngle;
  end;

TBooleanArray = Array of Boolean;

// ·ϵMS
TCreateObstructionOperation = class(TBaseOperation)
  private
    FRoadId: Integer;
    FDistance: Double;
    FLength: Double;
    FLeadIn: Double;
    FLeadOut: Double;
    FSpeedLimit: Integer;
    FLeftBlockedLane: TBooleanArray;
    FRightBlockedLane: TBooleanArray;
  public
    constructor Create; overload;
    constructor Create(roadId, speedLimit: Integer; distance,length,leadIn,leadOut: Double;
                        leftBlockedLane,rightBlockedLane: TBooleanArray); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property RoadId: Integer read FRoadId write FRoadId;
    property Distance: Double read FDistance  write FDistance;
    property Length: Double read FLength write FLength;
    property LeadIn: Double read FLeadIn write FLeadIn;
    property LeadOut: Double read FLeadOut write FLeadOut;
    property SpeedLimit: Integer read FSpeedLimit write FSpeedLimit;
    property LeftBlockedLane: TBooleanArray read FLeftBlockedLane write FLeftBlockedLane;
    property RightBlockedLane: TBooleanArray read FRightBlockedLane write FRightBlockedLane;
  end;

// ɾ·ϵMS
// Ա
// =========FObstructionIndexɾ·ϵindexϢ
// =========FRoadIdɾ·ϵĵ·Ϣ
TDeleteObstructionOperation = class(TBaseOperation)
  private
    FObstructionIndex: Integer;
    FRoadId: Integer;
  public
    constructor Create; overload;
    constructor Create(obstructionIndex, RoadId: Integer); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property ObstructionIndex: Integer read FObstructionIndex write FObstructionIndex;
    property RoadId: Integer read FRoadId write FRoadId;
  end;

// ͬƶϢMS
TSyncMovingInstanceOperation = class(TBaseOperation)
  private
    FModelIndex: Integer;
    FInstanceId: Integer;
    FIsCreate: Boolean;
    FPosition: GLPointType;
    FYawAngle: Double;
    FRollAngle: Double;
    FPitchAngle: Double;
  public
    constructor Create; overload;
    constructor Create(modelIndex, instanceId: Integer; position: GLPointType; yawAngle, rollAngle, pitchAngle: Double; isCreate: Boolean); overload;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Validate; override;

    property ModelIndex: Integer read FModelIndex write FModelIndex;
    property InstanceId: Integer read FInstanceId write FInstanceId;
    property Position: GLPointType read FPosition write FPosition;
    property YawAngle: Double read FYawAngle write FYawAngle;
    property RollAngle: Double read FRollAngle write FRollAngle;
    property PitchAngle: Double read FPitchAngle write FPitchAngle;
    property IsCreate: Boolean read FIsCreate write FIsCreate;
  end;

function searchModelInstanceIndexById(id: Integer): Integer;
function searchParticleGeneratorInstanceIndexById(id: Integer): Integer;
function searchModelInstanceIndexByName(name: String): Integer;
function searchRoadIndex(id: Integer): Integer;
function searchParticleGeneratorInstanceIndexByName(name: String): Integer;
function searchRoadModelInstanceIndexById(id: Integer): Integer;

implementation

uses
  ModelSyncPluginMain,
  vcl.dialogs,
  sysutils;

// Ĺ캯
constructor TBaseOperation.Create;
  begin
    Inherited Create;
    FSerial:= MODEL_SYNC_OPERATION;
    FValid:= false;
    FType:= MODEL_SYNC_NONE;
    FFlag:= 0;
    FId:= 0;
  end;

destructor TBaseOperation.Destroy;
  begin
    FSerial:= 0;
    FValid:= false;
    FType:= 0;
    FFlag:= 0;
    FId:= 0;
    Inherited Destroy;
  end;

// ½Ĺ캯
constructor TCreateModelOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_CREATE_MODEL;
    FModelIndex:= -1;
  end;

constructor TCreateModelOperation.Create(position: GLPointType; model: Integer);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_CREATE_MODEL;
    FPosition:= position;
    FModelIndex:= model;
  end;

destructor TCreateModelOperation.Destroy;
  begin
    Inherited Destroy;
    FreeAndNil(FPosition);
    FModelIndex:= 0;
  end;

// ½Ĺ캯
constructor TCreateFireOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_CREATE_FIRE;
    FModelIndex:= -1;
  end;

constructor TCreateFireOperation.Create(position: GLPointType; model: Integer);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_CREATE_FIRE;
    FPosition:= position;
    FModelIndex:= model;
  end;

destructor TCreateFireOperation.Destroy;
  begin
    Inherited Destroy;
    FreeAndNil(FPosition);
    FModelIndex:= 0;
  end;

// Ĺ캯
constructor TCommandOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_COMMAND;
    FModelInstance:= 0;
    FType:= 0;
    FCommandIndex:= 0;
    FModelInstanceType:= 0;
  end;

constructor TCommandOperation.Create(data: Byte; instanceType: Byte; modelInstance: Integer; commandIndex: Integer);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_COMMAND;
    FModelInstance:= modelInstance;
    FType:= data;
    FCommandIndex:= commandIndex;
    FModelInstanceType:= instanceType;
  end;

destructor TCommandOperation.Destroy;
  begin
    Inherited Destroy;
    FModelInstance:= 0;
    FType:= 0;
    FCommandIndex:= 0;
    FModelInstanceType:= 0;
  end;

// ɾĹ캯
constructor TDeleteModelOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_DELETE_MODEL;
    FModelInstance:= -1;
    FModelInstanceType:= 0;
  end;

constructor TDeleteModelOperation.Create(instance: Integer; instanceType: Byte);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_DELETE_MODEL;
    FModelInstance:= instance;
    FModelInstanceType:= instanceType;
  end;

destructor TDeleteModelOperation.Destroy;
  begin
    Inherited Destroy;
    FModelInstance:= 0;
    FModelInstanceType:= 0;
  end;

// תĹ캯
constructor TRotateModelOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_ROTATE_MODEL;
    FModelInstance:= -1;
    FModelInstanceType:= 0;
    FYawAngle:= 0.0;
  end;

constructor TRotateModelOperation.Create(instance: Integer; instanceType: Byte; yawAngle: Double);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_ROTATE_MODEL;
    FModelInstance:= instance;
    FModelInstanceType:= instanceType;
    FYawAngle:= yawAngle;
  end;

destructor TRotateModelOperation.Destroy;
  begin
    Inherited Destroy;
    FModelInstance := 0;
    FModelInstanceType:= 0;
    FYawAngle:= 0.0;
  end;

// ·ϲĹ캯
constructor TCreateObstructionOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_CREATE_OBSTRUCTION;
    FRoadId:= -1;
    FDistance:= 0.0;
    FLength:= 0.0;
    FLeadIn:= 0.0;
    FLeadOut:= 0.0;
    FSpeedLimit:= 0;
    SetLength(FLeftBlockedLane, 0);
    SetLength(FRightBlockedLane, 0);
  end;

constructor TCreateObstructionOperation.Create(roadId: Integer; speedLimit: Integer; distance: Double; length: Double; leadIn: Double; leadOut: Double; leftBlockedLane: TBooleanArray; rightBlockedLane: TBooleanArray);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_CREATE_OBSTRUCTION;
    FRoadId:= roadId;
    FDistance:= distance;
    FLength:= length;
    FLeadIn:= leadIn;
    FLeadOut:= leadOut;
    FSpeedLimit:= speedLimit;
    FLeftBlockedLane:= leftBlockedLane;
    FRightBlockedLane:= rightBlockedLane;
  end;

destructor TCreateObstructionOperation.Destroy;
  begin
    Inherited Destroy;
    FRoadId:= 0;
    FDistance:= 0.0;
    FLength:= 0.0;
    FLeadIn:= 0.0;
    FLeadOut:= 0.0;
    FSpeedLimit:= 0;
    SetLength(FLeftBlockedLane, 0);
    SetLength(FRightBlockedLane, 0);
  end;

// ɾ·ϲĹ캯
constructor TDeleteObstructionOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_DELETE_OBSTRUCTION;
    FObstructionIndex:= -1;
    FRoadId:= -1;
  end;

constructor TDeleteObstructionOperation.Create(obstructionIndex: Integer; RoadId: Integer);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_DELETE_OBSTRUCTION;
    FObstructionIndex:= obstructionIndex;
    FRoadId:= RoadId;
  end;

destructor TDeleteObstructionOperation.Destroy;
  begin
    Inherited Destroy;
    FObstructionIndex:= 0;
    FRoadId:= 0;
  end;

// ͬƶģͲĹ캯
constructor TSyncMovingInstanceOperation.Create;
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_SYNC_MOVING_INSTANCE;
    FModelIndex:= -1;
    FIsCreate:= false;
    FYawAngle:= 0.0;
    FPitchAngle:= 0.0;
    FRollAngle:= 0.0;
    FInstanceId:= -1;
  end;

constructor TSyncMovingInstanceOperation.Create(modelIndex: Integer; instanceId: Integer; position: GLPointType; yawAngle: Double; rollAngle: Double; pitchAngle: Double; isCreate: Boolean);
  begin
    Inherited Create;
    OperType:= MODEL_SYNC_SYNC_MOVING_INSTANCE;
    FModelIndex:= modelIndex;
    FYawAngle:= yawAngle;
    FPitchAngle:= pitchAngle;
    FRollAngle:= rollAngle;
    FInstanceId:= instanceId;
    FPosition:= position;
    FIsCreate:= isCreate;
  end;

destructor TSyncMovingInstanceOperation.Destroy;
  begin
    Inherited Destroy;
    FModelIndex:= 0;
    FYawAngle:= 0.0;
    FPitchAngle:= 0.0;
    FRollAngle:= 0.0;
    FInstanceId:= 0;
    FIsCreate:= false;
  end;

// ֤ǷһЧĲ
// Ҫӵ֤ЧԴ
// -1֤
// Ϊյ֤
// ڲ͵֤
// IDתindexǷ񳬳߽
procedure TCreateModelOperation.Validate;
  begin
    if FModelIndex <= 0 then begin
      Valid:= false;
      Exit;
    end;
    Valid:= true;
  end;

procedure TCreateFireOperation.Validate;
  begin
    if FModelIndex <= 0 then begin
      Valid:= false;
      Exit;
    end;
    Valid:= true;
  end;

procedure TCommandOperation.Validate;
  var
    project: IF8ProjectForRoad;
    maxIndex: Integer;
    instanceIndex: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    if FModelInstanceType = 1 then begin
      instanceIndex:= searchModelInstanceIndexById(FModelInstance);
      maxIndex:= project.numberOfModels;
    end
    else if FModelInstanceType = 2 then begin
      instanceIndex:= searchRoadModelInstanceIndexById(FModelInstance);
      maxIndex:= project.numberOfRoadModelInstance;
    end;
    if (instanceIndex = -1) or (instanceIndex > maxIndex) then begin
      Valid:= false;
      Exit;
    end;
    Valid:= true;
  end;

procedure TDeleteModelOperation.Validate;
  var
    project: IF8ProjectForRoad;
    maxIndex: Integer;
    instanceIndex: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    if FModelInstanceType = 1 then begin
      instanceIndex:= searchModelInstanceIndexById(FModelInstance);
      maxIndex:= project.numberOfModels;
    end
    else if FModelInstanceType = 2 then begin
      instanceIndex:= searchModelInstanceIndexByName(IntToStr(FModelInstance));
      maxIndex:= project.numberOfModels;
    end
    else if FModelInstanceType = 3 then begin
      instanceIndex:= searchParticleGeneratorInstanceIndexById(FModelInstance);
      maxIndex:= project.numberOfParticleGeneratorInstances;
    end
    else if FModelInstanceType = 4 then begin
      instanceIndex:= searchParticleGeneratorInstanceIndexByName(IntToStr(FModelInstance));
      maxIndex:= project.numberOfParticleGeneratorInstances;
    end
    else if FModelInstanceType = 5 then begin
      instanceIndex:= searchRoadModelInstanceIndexById(FModelInstance);
      maxIndex:= project.numberOfRoadModelInstance;
    end;
    if (instanceIndex = -1) or (instanceIndex > maxIndex) then begin
      Valid:= false;
      Exit;
    end;
    Valid:= true;
  end;

procedure TRotateModelOperation.Validate;
  var
    project: IF8ProjectForRoad;
    maxIndex: Integer;
    instanceIndex: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    if FModelInstanceType = 1 then begin
      instanceIndex:= searchModelInstanceIndexById(FModelInstance);
      maxIndex:= project.numberOfModels;
    end
    else if FModelInstanceType = 2 then begin
      instanceIndex:= searchModelInstanceIndexByName(IntToStr(FModelInstance));
      maxIndex:= project.numberOfModels;
    end;
    if (instanceIndex = -1) or (instanceIndex > maxIndex) then begin
      Valid:= false;
      Exit;
    end;
    Valid:= true;
  end;

procedure TCreateObstructionOperation.Validate;
  var
    road: IF8Road;
    index: Integer;
  begin
    index:= searchRoadIndex(FRoadId);
    if index = -1 then begin
      Valid:= false;
      Exit;
    end;
    Valid:= true;
  end;

procedure TDeleteObstructionOperation.Validate;
  var
    road: IF8Road;
    project: IF8ProjectForRoad;
    index: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    index:= searchRoadIndex(FRoadId);
    if index = -1 then begin
      Valid:= false;
      Exit;
    end;
    if Assigned(project) then begin
      road:= project.road[index];
    end;
    if ((FObstructionIndex <= 0) or (FObstructionIndex > road.numberOfObstructions)) then begin
      Valid:= false;
      Exit;
    end;
    Valid:= true;
  end;

procedure TSyncMovingInstanceOperation.Validate;
  var
    project: IF8ProjectForRoad;
    index: Integer;
  begin
    if FModelIndex <= 0 then begin
      Valid:= false;
      Exit;
    end;
    project:= ModelSyncPlugin.winRoadApplication.project;
    index:= searchModelInstanceIndexByName(IntToStr(FInstanceId));
    if index <> -1 then begin
      FIsCreate:= true;
    end;
    Valid:= true;
  end;

// ʵִеĺ
procedure TCreateModelOperation.Execute;
  var
    project: IF8ProjectForRoad;
    currentModelInstance: IF8ModelInstance;
  begin
    if Valid then begin
      project := ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        currentModelInstance:= project.MakeModel(project.threeDModel[FModelIndex], FPosition[_x], FPosition[_y], FPosition[_z]);
        currentModelInstance.name:= IntToStr(Id);
        ModelSyncPlugin.winRoadApplication.mayHaveChanged := true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

procedure TCreateFireOperation.Execute;
  var
    project: IF8ProjectForRoad;
    currentModelInstance: IF8FireInstance;
  begin
    if Valid then begin
      project := ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        currentModelInstance:= project.MakeFire(FModelIndex,FPosition[_x],FPosition[_z]);
        currentModelInstance.instancePosition:= FPosition;
        currentModelInstance.name:= IntToStr(Id);
        ModelSyncPlugin.winRoadApplication.mayHaveChanged := true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

procedure TCommandOperation.Execute;
  var
    project: IF8ProjectForRoad;
    index: Integer;
  begin
    if Valid then begin
      project := ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        if FModelInstanceType = 1 then begin
          index:= searchModelInstanceIndexById(FModelInstance);
          if FType = 1 then begin
            project.modelInstance[index].ActivateCommand(project.modelInstance[index].commandName[FCommandIndex]);
          end
          else if FType = 2 then begin
            project.modelInstance[index].DeactivateCommand(project.modelInstance[index].commandName[FCommandIndex]);
          end;
        end
        else if FModelInstanceType = 2 then begin
          index:= searchRoadModelInstanceIndexById(FModelInstance);
          if FType = 1 then begin
            project.roadmodelInstance[index].ActivateCommand(project.roadmodelInstance[index].commandName[FCommandIndex]);
          end
          else if FType = 2 then begin
            project.roadmodelInstance[index].DeactivateCommand(project.roadmodelInstance[index].commandName[FCommandIndex]);
          end;
        end;
        ModelSyncPlugin.winRoadApplication.mayHaveChanged := true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

procedure TDeleteModelOperation.Execute;
  var
    project: IF8ProjectForRoad;
    index: Integer;
  begin
    if Valid then begin
      project := ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        if FModelInstanceType = 1 then begin
          index:= searchModelInstanceIndexById(FModelInstance);
          project.DeleteModelInstance(index);
        end
        else if FModelInstanceType = 2 then begin
          index:= searchModelInstanceIndexByName(IntToStr(FModelInstance));
          project.DeleteModelInstance(index);
        end
        else if FModelInstanceType = 3 then begin
          index:= searchParticleGeneratorInstanceIndexById(FModelInstance);
          project.ParticleGeneratorInstance[index].objectVisibility:= false;
        end
        else if FModelInstanceType = 4 then begin
          index:= searchParticleGeneratorInstanceIndexByName(IntToStr(FModelInstance));
          project.ParticleGeneratorInstance[index].objectVisibility:= false;
        end
        else if FModelInstanceType = 5 then begin
          index:= searchRoadModelInstanceIndexById(FModelInstance);
          project.DeleteRoadModelInstance(index);
        end;
        ModelSyncPlugin.winRoadApplication.mayHaveChanged := true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

procedure TRotateModelOperation.Execute;
  var
    project: IF8ProjectForRoad;
    index: Integer;
  begin
    if Valid then begin
      project := ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        if FModelInstanceType = 1 then begin
          index:= searchModelInstanceIndexById(FModelInstance);
          project.ModelInstance[index].yawAngle:= FYawAngle;
        end
        else if FModelInstanceType = 2 then begin
          index:= searchModelInstanceIndexByName(IntToStr(FModelInstance));
          project.ModelInstance[index].yawAngle:= FYawAngle;
        end;
        ModelSyncPlugin.winRoadApplication.mayHaveChanged := true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

procedure TCreateObstructionOperation.Execute;
  var
    project: IF8ProjectForRoad;
    roadIndex: Integer;
    obstruction: IF8RoadObstruction;
    i: Integer;
  begin
    if Valid then begin
      project := ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        roadIndex:= searchRoadIndex(FRoadId);
        obstruction:= project.road[roadIndex].CreateObstructionEx(FDistance,FLength, FLeadIn, FLeadOut, FSpeedLimit);
        for i := 1 to obstruction.GetRightLanes do
          obstruction.rightBlockedLane[i] := FLeftBlockedLane[i - 1];
        for i := 1 to obstruction.GetLeftLanes do
          obstruction.leftBlockedLane[i] := FRightBlockedLane[i - 1];
        //obstruction.displayType:=  _DisplayLaneTexture;
        obstruction.Apply;
        project.road[roadIndex].CheckObstructions;

        ModelSyncPlugin.winRoadApplication.mayHaveChanged:= true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

procedure TDeleteObstructionOperation.Execute;
  var
    project: IF8ProjectForRoad;
    roadIndex: Integer;
  begin
    if Valid then begin
      project := ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        roadIndex:= searchRoadIndex(FRoadId);
        project.road[roadIndex].DeleteObstruction(FObstructionIndex);
        ModelSyncPlugin.winRoadApplication.mayHaveChanged := true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

procedure TSyncMovingInstanceOperation.Execute;
  var
    project: IF8ProjectForRoad;
    currentModelInstance: IF8ModelInstance;
    index: Integer;
  begin
    Validate;
    if Valid then begin
      project:= ModelSyncPlugin.winRoadApplication.project;
      if Assigned(project) then begin
        if not FIsCreate then begin
          currentModelInstance:= project.MakeModel(project.threeDModel[FModelIndex], FPosition[_x], FPosition[_y], FPosition[_z]);
          currentModelInstance.name:= IntToStr(FInstanceId);
        end
        else begin
          index:= searchModelInstanceIndexByName(IntToStr(FInstanceId));
          currentModelInstance:= project.modelInstance[index];
          currentModelInstance.instancePosition:= FPosition;
        end;
        currentModelInstance.rollAngle:= FRollAngle;
        currentModelInstance.yawAngle:= FYawAngle;
        currentModelInstance.pitchAngle:= FPitchAngle;
        ModelSyncPlugin.winRoadApplication.mayHaveChanged := true;
        ModelSyncPlugin.winRoadApplication.mainForm.openGL.Refresh;
      end;
    end;
  end;

// ģ͵nameѰģ͵ֵ
function searchModelInstanceIndexByName(name: String): Integer;
  var
    project: IF8ProjectForRoad;
    i: Integer;
    low: Integer;
    high: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    low:= 1;
    high:= project.numberOfModels;
    if Assigned(project) then begin
      for I := Low to High do begin
        if (project.modelInstance[i].name = name) then begin
          Result:= i;
          Exit;
        end;
      end;
    end;
    result:= -1;
  end;

// ģ͵idѰģ͵ֵ
function searchModelInstanceIndexById(id: Integer): Integer;
  var
    project: IF8ProjectForRoad;
    k: Integer;
    i: Integer;
    low: Integer;
    high: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    low:= 1;
    high:= project.numberOfModels;
    if Assigned(project) then begin
      for I := Low to High do begin
        if (project.modelInstance[i].id = id) then begin
          Result:= i;
          Exit;
        end;
      end;
    end;
    result:= -1;
  end;

// ģ͵idѰģ͵ֵ
function searchParticleGeneratorInstanceIndexById(id: Integer): Integer;
  var
    project: IF8ProjectForRoad;
    k: Integer;
    i: Integer;
    low: Integer;
    high: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    low:= 1;
    high:= project.numberOfParticleGeneratorInstances;
    if Assigned(project) then begin
      for I := Low to High do begin
        if (project.particleGeneratorInstance[i].id = id) then begin
          Result:= i;
          Exit;
        end;
      end;
    end;
    result:= -1;
  end;

// ģ͵nameѰģ͵ֵ
function searchParticleGeneratorInstanceIndexByName(name: String): Integer;
  var
    project: IF8ProjectForRoad;
    i: Integer;
    low: Integer;
    high: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    low:= 1;
    high:= project.numberOfParticleGeneratorInstances;
    if Assigned(project) then begin
      for I := Low to High do begin
        if (project.particleGeneratorInstance[i].name = name) then begin
          Result:= i;
          Exit;
        end;
      end;
    end;
    result:= -1;
  end;

// ݵ·ģ͵idѰģ͵ֵ
function searchRoadModelInstanceIndexById(id: Integer): Integer;
  var
    project: IF8ProjectForRoad;
    k: Integer;
    i: Integer;
    low: Integer;
    high: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    low:= 1;
    high:= project.numberOfRoadModelInstance;
    if Assigned(project) then begin
      for I := Low to High do begin
        if (project.roadmodelInstance[i].id = id) then begin
          Result:= i;
          Exit;
        end;
      end;
    end;
    result:= -1;
  end;

// ݵ·idѰҵ·ֵ
function searchRoadIndex(id: Integer): Integer;
  var
    project: IF8ProjectForRoad;
    k: Integer;
    i: Integer;
    low: Integer;
    high: Integer;
  begin
    project:= ModelSyncPlugin.winRoadApplication.project;
    low:= 1;
    high:= project.numberOfRoads;
    if Assigned(project) then begin
      if project.road[low].ID = id then begin
        result:= low;
        exit;
      end;
      if project.road[high].ID = id then begin
        result:= high;
        exit;
      end;
      for i := Low to High do
      begin
        k:= (low + high) div 2;
        if (project.road[k].ID = id) then
          begin
            result:= k;
            exit;
          end
        else if (project.road[k].ID < id) then
          begin
            Low:= k;
          end
        else if (project.road[k].ID > id) then
          begin
            High:= k;
          end;
      end;
      result:= -1;
    end;
  end;

end.
