unit MS_Parser;

interface

uses
  MS_Operation;

type

TByteArray = Array[0..99] of Byte;

TMSOperationParser = class
  private
    FCurrentOperation: TBaseOperation;
    FBuffer: TByteArray;
  public
    constructor Create;
    destructor Destroy;
    procedure ToByteArray(operation: TBaseOperation);
    procedure ToOperation(buf: Array of Byte);

    property Buffer: TByteArray read FBuffer;
    property CurrentOperation: TBaseOperation read FCurrentOperation;
  end;

implementation

uses
  F8OpenGL,
  F8GLUtils,
  SysUtils;

constructor TMSOperationParser.Create;
  begin
    Inherited Create;
    FCurrentOperation:= nil;
    FillChar(FBuffer,sizeof(FBuffer),0);
  end;

destructor TMSOperationParser.Destroy;
  begin
    Inherited Destroy;
    FreeAndNil(FCurrentOperation);
    FreeAndNil(FBuffer);
  end;

procedure TMSOperationParser.ToByteArray(operation: TBaseOperation);
  var
    DataLength: Integer;
    I: Integer;
  begin
    if Assigned(operation) then begin
      FBuffer[0]:= operation.Serial;
      FBuffer[1]:= operation.OperType;
      if operation.Valid then
        FBuffer[2]:= 1
      else
        FBuffer[2]:= 0;
      Move(operation.Id, FBuffer[3], 4);

      if (operation is TCreateModelOperation) then begin
        Move((operation as TCreateModelOperation).Position[_x], FBuffer[7],4);
        Move((operation as TCreateModelOperation).Position[_y], FBuffer[11],4);
        Move((operation as TCreateModelOperation).Position[_z], FBuffer[15],4);
        Move((operation as TCreateModelOperation).Position[_w], FBuffer[19],4);
        Move((operation as TCreateModelOperation).ModelIndex, FBuffer[23],4);
        FBuffer[27]:= operation.Flag;
      end

      else if (operation is TCreateFireOperation) then begin
        Move((operation as TCreateFireOperation).Position[_x], FBuffer[7],4);
        Move((operation as TCreateFireOperation).Position[_y], FBuffer[11],4);
        Move((operation as TCreateFireOperation).Position[_z], FBuffer[15],4);
        Move((operation as TCreateFireOperation).Position[_w], FBuffer[19],4);
        Move((operation as TCreateFireOperation).ModelIndex, FBuffer[23],4);
        FBuffer[27]:= operation.Flag;
      end

      else if (operation is TCommandOperation) then begin
        FBuffer[7]:= (operation as TCommandOperation).CommandType;
        FBuffer[8]:= (operation as TCommandOperation).ModelInstanceType;
        Move((operation as TCommandOperation).ModelInstance, FBuffer[9], 4);
        Move((operation as TCommandOperation).CommandIndex, FBuffer[13], 4);
        FBuffer[17]:= operation.Flag;
      end

      else if (operation is TDeleteModelOperation) then begin
        FBuffer[7]:= (operation as TDeleteModelOperation).ModelInstanceType;
        Move((operation as TDeleteModelOperation).ModelInstance, FBuffer[8],4);
        FBuffer[12]:= operation.Flag;
      end

      else if (operation is TRotateModelOperation) then begin
        FBuffer[7]:= (operation as TRotateModelOperation).ModelInstanceType;
        Move((operation as TRotateModelOperation).ModelInstance, FBuffer[8],4);
        Move((operation as TRotateModelOperation).YawAngle, FBuffer[12],8);
        FBuffer[20]:= operation.Flag;
      end

      else if (operation is TCreateObstructionOperation) then begin
        Move((operation as TCreateObstructionOperation).RoadId, FBuffer[7], 4);
        Move((operation as TCreateObstructionOperation).Distance, FBuffer[11], 8);
        Move((operation as TCreateObstructionOperation).Length, FBuffer[19], 8);
        Move((operation as TCreateObstructionOperation).LeadIn, FBuffer[27], 8);
        Move((operation as TCreateObstructionOperation).LeadOut, FBuffer[35], 8);
        Move((operation as TCreateObstructionOperation).SpeedLimit, FBuffer[43], 4);
        for I := 0 to 3 do begin
          if (operation as TCreateObstructionOperation).LeftBlockedLane[i] then
            FBuffer[47 + i]:= 1
          else
            FBuffer[47 + i]:= 0;
        end;
        for I := 0 to 3 do begin
          if (operation as TCreateObstructionOperation).RightBlockedLane[i] then
            FBuffer[51 + i]:= 1
          else
            FBuffer[51 + i]:= 0;
        end;
        FBuffer[55]:= operation.Flag;
      end

      else if (operation is TDeleteObstructionOperation) then begin
        Move((operation as TDeleteObstructionOperation).ObstructionIndex, FBuffer[7], 4);
        Move((operation as TDeleteObstructionOperation).RoadId, FBuffer[11], 4);
        FBuffer[15]:= operation.Flag;
      end

      else if (operation is TSyncMovingInstanceOperation) then begin
        Move((operation as TSyncMovingInstanceOperation).Position[_x], FBuffer[7],4);
        Move((operation as TSyncMovingInstanceOperation).Position[_y], FBuffer[11],4);
        Move((operation as TSyncMovingInstanceOperation).Position[_z], FBuffer[15],4);
        Move((operation as TSyncMovingInstanceOperation).Position[_w], FBuffer[19],4);
        Move((operation as TSyncMovingInstanceOperation).ModelIndex, FBuffer[23],4);
        Move((operation as TSyncMovingInstanceOperation).InstanceId, FBuffer[27],4);
        Move((operation as TSyncMovingInstanceOperation).YawAngle, FBuffer[31],8);
        Move((operation as TSyncMovingInstanceOperation).RollAngle, FBuffer[39],8);
        Move((operation as TSyncMovingInstanceOperation).PitchAngle, FBuffer[47],8);
        if (operation as TSyncMovingInstanceOperation).IsCreate then
          FBuffer[55]:= 1
        else
          FBuffer[55]:= 0;
        FBuffer[56]:= operation.Flag;
      end;

    end;
  end;

procedure TMSOperationParser.ToOperation(buf: Array of Byte);
  var
    pos: GLPointType;
    tmpLBooleanArray: TBooleanArray;
    tmpRBooleanArray: TBooleanArray;
  begin

    if buf[1] = MODEL_SYNC_CREATE_MODEL then begin
      FCurrentOperation:= TCreateModelOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      pos[_x]:= PSingle(@buf[7])^;
      pos[_y]:= PSingle(@buf[11])^;
      pos[_z]:= PSingle(@buf[15])^;
      pos[_w]:= PSingle(@buf[19])^;
      (FCurrentOperation as TCreateModelOperation).Position:= pos;
      (FCurrentOperation as TCreateModelOperation).ModelIndex:= PInteger(@buf[23])^;
      FCurrentOperation.Flag:= buf[27];
    end

    else if buf[1] = MODEL_SYNC_CREATE_FIRE then begin
      FCurrentOperation:= TCreateFireOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      pos[_x]:= PSingle(@buf[7])^;
      pos[_y]:= PSingle(@buf[11])^;
      pos[_z]:= PSingle(@buf[15])^;
      pos[_w]:= PSingle(@buf[19])^;
      (FCurrentOperation as TCreateFireOperation).Position:= pos;
      (FCurrentOperation as TCreateFireOperation).ModelIndex:= PInteger(@buf[23])^;
      FCurrentOperation.Flag:= buf[27];
    end

    else if buf[1] = MODEL_SYNC_COMMAND then begin
      FCurrentOperation:= TCommandOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      (FCurrentOperation as TCommandOperation).ModelInstanceType:= buf[7];
      (FCurrentOperation as TCommandOperation).CommandType:= buf[8];
      (FCurrentOperation as TCommandOperation).ModelInstance:= PInteger(@buf[9])^;
      (FCurrentOperation as TCommandOperation).CommandIndex:= PInteger(@buf[13])^;
      FCurrentOperation.Flag:= buf[17];
    end

    else if buf[1] = MODEL_SYNC_DELETE_MODEL then begin
      FCurrentOperation:= TDeleteModelOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      (FCurrentOperation as TDeleteModelOperation).ModelInstanceType:= buf[7];
      (FCurrentOperation as TDeleteModelOperation).ModelInstance:= PInteger(@buf[8])^;
      FCurrentOperation.Flag:= buf[12];
    end

    else if buf[1] = MODEL_SYNC_ROTATE_MODEL then begin
      FCurrentOperation:= TRotateModelOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      (FCurrentOperation as TRotateModelOperation).ModelInstanceType:= buf[7];
      (FCurrentOperation as TRotateModelOperation).ModelInstance:= PInteger(@buf[8])^;
      (FCurrentOperation as TRotateModelOperation).YawAngle:= PDouble(@buf[12])^;
      FCurrentOperation.Flag:= buf[20];
    end

    else if buf[1] = MODEL_SYNC_CREATE_OBSTRUCTION then begin
      FCurrentOperation:= TCreateObstructionOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      (FCurrentOperation as TCreateObstructionOperation).RoadId:= PInteger(@buf[7])^;
      (FCurrentOperation as TCreateObstructionOperation).Distance:= PDouble(@buf[11])^;
      (FCurrentOperation as TCreateObstructionOperation).Length:= PDouble(@buf[19])^;
      (FCurrentOperation as TCreateObstructionOperation).LeadIn:= PDouble(@buf[27])^;
      (FCurrentOperation as TCreateObstructionOperation).LeadOut:= PDouble(@buf[35])^;
      (FCurrentOperation as TCreateObstructionOperation).SpeedLimit:= PInteger(@buf[43])^;
      SetLength(tmpLBooleanArray, 4);
      tmpLBooleanArray[0]:= (buf[47] = 1);
      tmpLBooleanArray[1]:= (buf[48] = 1);
      tmpLBooleanArray[2]:= (buf[49] = 1);
      tmpLBooleanArray[3]:= (buf[50] = 1);
      (FCurrentOperation as TCreateObstructionOperation).LeftBlockedLane:= tmpLBooleanArray;
      SetLength(tmpRBooleanArray, 4);
      tmpRBooleanArray[0]:= (buf[51] = 1);
      tmpRBooleanArray[1]:= (buf[52] = 1);
      tmpRBooleanArray[2]:= (buf[53] = 1);
      tmpRBooleanArray[3]:= (buf[54] = 1);
      (FCurrentOperation as TCreateObstructionOperation).RightBlockedLane:= tmpRBooleanArray;
      FCurrentOperation.Flag:= buf[55];
    end

    else if buf[1] = MODEL_SYNC_DELETE_OBSTRUCTION then begin
      FCurrentOperation:= TDeleteObstructionOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      (FCurrentOperation as TDeleteObstructionOperation).ObstructionIndex:= PInteger(@buf[7])^;
      (FCurrentOperation as TDeleteObstructionOperation).RoadId:= PInteger(@buf[11])^;
      FCurrentOperation.Flag:= buf[15];
    end

    else if buf[1] = MODEL_SYNC_SYNC_MOVING_INSTANCE then begin
      FCurrentOperation:= TSyncMovingInstanceOperation.Create;
      FCurrentOperation.Serial:= buf[0];
      FCurrentOperation.OperType:= buf[1];
      FCurrentOperation.Valid:= (buf[2] = 1);
      FCurrentOperation.Id:= PInteger(@buf[3])^;
      pos[_x]:= PSingle(@buf[7])^;
      pos[_y]:= PSingle(@buf[11])^;
      pos[_z]:= PSingle(@buf[15])^;
      pos[_w]:= PSingle(@buf[19])^;
      (FCurrentOperation as TSyncMovingInstanceOperation).Position:= pos;
      (FCurrentOperation as TSyncMovingInstanceOperation).ModelIndex:= PInteger(@buf[23])^;
      (FCurrentOperation as TSyncMovingInstanceOperation).InstanceId:= PInteger(@buf[27])^;
      (FCurrentOperation as TSyncMovingInstanceOperation).YawAngle:= PDouble(@buf[31])^;
      (FCurrentOperation as TSyncMovingInstanceOperation).RollAngle:= PDouble(@buf[39])^;
      (FCurrentOperation as TSyncMovingInstanceOperation).PitchAngle:= PDouble(@buf[47])^;
      (FCurrentOperation as TSyncMovingInstanceOperation).IsCreate:= (buf[55] = 1);
      FCurrentOperation.Flag:= buf[56];
    end;

  end;

end.
