unit ModelSyncCommandForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, PluginCore, F8GLUtils, F8OpenGL;

type
  TCommandEvent = procedure(modelInstance, commandIndex: Integer; modelInstanceType, commandType: Byte) of object;

  TModelInstance = class
  public
    instance: IF8ModelInstance;
  end;

  TRoadModelInstance = class
  public
    instance: IF8RoadModelInstance;
  end;

  TModelCommandForm = class(TForm)
    InstanceList: TListBox;
    CommandList: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    Activate: TButton;
    Deactivate: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure InstanceListClick(Sender: TObject);
    procedure InstanceListDbClick(Sender: TObject);
    procedure ActivateClick(Sender: TObject);
    procedure DeactivateClick(Sender: TObject);
  private
    { Private declarations }
    p_userSet : Boolean;
    p_eyePoint : GLPointType;
    p_viewPointPoint : GLPointType;
    p_rollAngle : Double;
    p_updatingControls : Boolean;
    FOnHandle: TCommandEvent;
  public
    { Public declarations }
    procedure SetupCamera;
    property onHandle: TCommandEvent read FOnHandle write FOnHandle;
  end;

var
  ModelCommandForm: TModelCommandForm;

implementation
uses
  ModelSyncPluginMain;
{$R *.dfm}

procedure TModelCommandForm.FormShow(Sender: TObject);
  var
    project: IF8ProjectForRoad;
    I: Integer;
    modelInstance: TModelInstance;
    roadModelInstance: TRoadModelInstance;
  begin
    Deactivate.Hide;
    project:= ModelSyncPlugin.winRoadApplication.project;
    for I := 1 to project.numberOfModels do begin
      if project.modelInstance[i].canControl then begin
        modelInstance:= TModelInstance.Create;
        modelInstance.instance:= project.modelInstance[i];
        InstanceList.AddItem(modelInstance.instance.name, modelInstance);
      end;
    end;
    for I := 1 to project.numberOfRoadModelInstance do begin
      if project.roadmodelInstance[i].canControl then begin
        roadModelInstance:= TRoadModelInstance.Create;
        roadModelInstance.instance:= project.roadmodelInstance[i];
        InstanceList.AddItem(roadModelInstance.instance.name, roadModelInstance);
      end;
    end;
  end;

procedure TModelCommandForm.InstanceListClick(Sender: TObject);
  var
    modelInstance: IF8ModelInstance;
    i: Integer;
  begin
    CommandList.Clear;
    if InstanceList.ItemIndex >= 0 then
      modelInstance:= TModelInstance(InstanceList.Items.Objects[InstanceList.ItemIndex]).instance;
    if Assigned(modelInstance) then begin
      for I := 1 to modelInstance.numberOfCommands do begin
        CommandList.AddItem(modelInstance.commandName[i],nil);
      end;
    end;
  end;

procedure TModelCommandForm.ActivateClick(Sender: TObject);
var
  modelInstance: IF8ModelInstance;
  roadModelInstance: IF8RoadModelInstance;
begin
  if (InstanceList.ItemIndex >= 0) and (CommandList.ItemIndex >= 0) then begin
    if InstanceList.Items.Objects[InstanceList.ItemIndex] is TModelInstance then begin
      modelInstance:= TModelInstance(InstanceList.Items.Objects[InstanceList.ItemIndex]).instance;
      if Assigned(FOnHandle) then begin
        FOnHandle(modelInstance.ID, (CommandList.ItemIndex + 1), 1, 1);
      end;
    end
    else if InstanceList.Items.Objects[InstanceList.ItemIndex] is TRoadModelInstance then begin
      roadModelInstance:= TRoadModelInstance(InstanceList.Items.Objects[InstanceList.ItemIndex]).instance;
      if Assigned(FOnHandle) then begin
        FOnHandle(roadmodelInstance.ID, (CommandList.ItemIndex + 1), 1, 2);
      end;
    end;
  end;
end;

procedure TModelCommandForm.DeactivateClick(Sender: TObject);
  var
    modelInstance: IF8ModelInstance;
    roadModelInstance: IF8RoadModelInstance;
  begin
    if (InstanceList.ItemIndex >= 0) and (CommandList.ItemIndex >= 0) then begin
      if InstanceList.Items.Objects[InstanceList.ItemIndex] is TModelInstance then begin
        modelInstance:= TModelInstance(InstanceList.Items.Objects[InstanceList.ItemIndex]).instance;
        if Assigned(FOnHandle) then begin
          FOnHandle(modelInstance.ID, (CommandList.ItemIndex + 1), 2, 1);
        end;
      end
      else if InstanceList.Items.Objects[InstanceList.ItemIndex] is TRoadModelInstance then begin
        roadModelInstance:= TRoadModelInstance(InstanceList.Items.Objects[InstanceList.ItemIndex]).instance;
        if Assigned(FOnHandle) then begin
          FOnHandle(roadmodelInstance.ID, (CommandList.ItemIndex + 1), 2, 2);
        end;
      end;
    end;
  end;

procedure TModelCommandForm.FormHide(Sender: TObject);
  begin
    InstanceList.Clear;
    CommandList.Clear;
  end;

procedure TModelCommandForm.SetupCamera;
  var
    angles : GLPointType;
  begin
    p_updatingControls := true;
    //must ovewrite this value each time or UC-win/Road will reset the angle to 0.
    ModelSyncPlugin.winRoadApplication.mainForm.openGL.cameraTilt := p_rollAngle;
    //If the user changed some values in the editors of this form.
    //(see TFormCameraControl.EditChange)
    //Overwrites the position of the camera.
    if p_userSet then begin
      p_userSet := false;
      ModelSyncPlugin.winRoadApplication.mainForm.openGL.Camera.Eye.x := p_eyePoint[_x];
      ModelSyncPlugin.winRoadApplication.mainForm.openGL.Camera.Eye.y := p_eyePoint[_y];
      ModelSyncPlugin.winRoadApplication.mainForm.openGL.Camera.Eye.z := p_eyePoint[_z];
      ModelSyncPlugin.winRoadApplication.mainForm.openGL.Camera.ViewPoint.x := p_viewPointPoint[_x];
      ModelSyncPlugin.winRoadApplication.mainForm.openGL.Camera.ViewPoint.y := p_viewPointPoint[_y];
      ModelSyncPlugin.winRoadApplication.mainForm.openGL.Camera.ViewPoint.z := p_viewPointPoint[_z];
    end;
    p_updatingControls := false;
  end;

procedure TModelCommandForm.InstanceListDbClick(Sender: TObject);
  var
    angles : GLPointType;
    yaw : Double;
    pitch : Double;
    modelInstance: IF8ModelInstance;
    roadModelInstance: IF8RoadModelInstance;
  begin
    //Does not update the camera while updating the controls
    if p_updatingControls then
      Exit;
    //Get camera parameters from the controls
    if InstanceList.Items.Objects[InstanceList.ItemIndex] is TModelInstance then begin
      modelInstance:= TModelInstance(InstanceList.Items.Objects[InstanceList.ItemIndex]).instance;
      p_eyePoint[_x] := modelInstance.instancePosition[_x] - 20;
      p_eyePoint[_y] := modelInstance.instancePosition[_y] + 40;
      p_eyePoint[_z] := modelInstance.instancePosition[_z] - 20;
      p_viewPointPoint := modelInstance.instancePosition;
    end else
    if InstanceList.Items.Objects[InstanceList.ItemIndex] is TRoadModelInstance then begin
      roadModelInstance:= TRoadModelInstance(InstanceList.Items.Objects[InstanceList.ItemIndex]).instance;
      if pos('',roadModelInstance.name) <> 0 then begin
        p_eyePoint[_x] := roadModelInstance.instancePosition[_x] - 20;
        p_eyePoint[_y] := roadModelInstance.instancePosition[_y] + 20;
        p_eyePoint[_z] := roadModelInstance.instancePosition[_z] - 20;
      end else begin
        p_eyePoint[_x] := roadModelInstance.instancePosition[_x] + 20;
        p_eyePoint[_y] := roadModelInstance.instancePosition[_y] + 20;
        p_eyePoint[_z] := roadModelInstance.instancePosition[_z] + 20;
      end;
      p_viewPointPoint := roadModelInstance.instancePosition;
    end;
    //Set flag to true in order to update the camera when UC-win/Road requires it.
    p_userSet := true;
    p_rollAngle := 0;
  end;

end.
