unit ModelSyncPluginMain;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  VCL.Dialogs,
  VCL.Forms,
  PluginCore,
  MS_ClientForm,
  MS_ServerForm,
  F8OpenGL,
  ModelSyncCommandForm,
  MS_ThreeDeeTextForm,
  ChooseThreeDeeModelForm,
  ConfigRoadObstructionForm,
  DeleteRoadObstructionForm,
  ModelSyncResource;

type
//=== need update ==============================================================
TModelSyncPlugin = class(TF8PLuginClass, IF8OpenGLPlugin)
  private
    p_winRoadApplication : IF8ApplicationServices;
    p_ribbon: IF8Ribbon;
    p_ribbonTab: IF8RibbonTab;
    p_ribbonGroup: IF8RibbonGroup;

    procedure PaintScene(const opengl : TF8OpenGL);
    procedure PaintHUD(const opengl : TF8OpenGL);
    procedure SetupCamera(const opengl : TF8OpenGL);

    procedure CreateRibbons;
    procedure FreeRibbons;
    procedure AbleMenus(enable : Boolean);
    procedure ModelSyncClientClick(Sender: TObject);
    procedure ModelSyncServerClick(Sender: TObject);
    procedure ModelCommandFormClick(Sender: TObject);
    procedure ThreeDeeTextFormClick(Sender: TObject);
    procedure ProjectDestroyProc;
    procedure RemoveForms;
  public
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;

    property winRoadApplication : IF8ApplicationServices read p_winRoadApplication;
  end;

//Call back functions
procedure RegisterUserPlugin(out optionName, apiVersion, Copyright : String);
procedure LoadPlugin;
procedure UnloadPlugin;

var
  ModelSyncPlugin : TModelSyncPlugin;

implementation

//=== finish ===================================================================
procedure RegisterUserPlugin(out optionName, apiVersion, Copyright : String);
  begin
    optionName := 'Model Sync Plugin';
    apiVersion := PLUGIN_VERSION;
	  Copyright := 'ZHANG ZHE TAO';
  end;

//=== finish ===================================================================
procedure LoadPlugin;
  begin
    if not Assigned(ModelSyncPlugin) then
      ModelSyncPlugin := TModelSyncPlugin.Create;

    if Assigned(ModelSyncPlugin) and Assigned(ModelSyncPlugin.winRoadApplication) then
      ModelSyncPlugin.winRoadApplication.RegisterPluginObject(ModelSyncPlugin);
  end;

//=== finish ===================================================================
procedure UnloadPlugin;
  begin
    if Assigned(ModelSyncPlugin) and Assigned(ModelSyncPlugin.winRoadApplication) then
      ModelSyncPlugin.winRoadApplication.UnRegisterPluginObject(ModelSyncPlugin);
    ModelSyncPlugin.Free;
    ModelSyncPlugin := nil;
  end;

procedure TModelSyncPlugin.AfterConstruction;
  var
    method : TMethod;
  begin
    inherited;
    Supports(ApplicationServices, IF8ApplicationServices,p_winRoadApplication);
    CreateRibbons;
    PluginAbleMenusProc(method) := AbleMenus;
    p_winRoadApplication.RegisterEventHandler(_plgPluginAbleMenus, method);

    PluginNotifyProc(method):= ProjectDestroyProc;
    p_winRoadApplication.RegisterEventHandler(_plgProjectDestroy,method);
  end;

procedure TModelSyncPlugin.BeforeDestruction;
  var
    method : TMethod;
  begin
    //Free Ribbons
    FreeRibbons;

    PluginAbleMenusProc(method) := AbleMenus;
    p_winRoadApplication.UnRegisterEventHandler(_plgPluginAbleMenus, method);

    PluginNotifyProc(method):= ProjectDestroyProc;
    p_winRoadApplication.UnRegisterEventHandler(_plgProjectDestroy,method);

    RemoveForms;

    //Release Entry point
    p_winRoadApplication := nil;
    inherited;
  end;

//=== finish ===================================================================
procedure TModelSyncPlugin.AbleMenus(enable: Boolean);
  begin
    if Assigned(FormModelSyncResource) then begin
      FormModelSyncResource.ButtonModelSyncClient.Enabled:= enable;
      FormModelSyncResource.ButtonModelSyncServer.Enabled:= enable;
    end;
  end;

//=== finish ===================================================================
procedure TModelSyncPlugin.FreeRibbons;
  begin
    if Assigned(FormModelSyncResource) and Assigned(p_RibbonGroup) then
      begin
        p_RibbonGroup.RemoveGroupControl(FormModelSyncResource.MainPanel);
        FreeAndNil(FormModelSyncResource);
      end;
    if Assigned(p_RibbonTab) then
      begin
        p_RibbonTab.DeleteGroup(p_RibbonGroup);
        p_RibbonGroup := nil;
      end;
    p_RibbonTab := nil;
    p_Ribbon := nil;
  end;

//=== finish ===================================================================
procedure TModelSyncPlugin.CreateRibbons;
  const
    PLUGIN_TABCONTROLNAME = 'ModelSyncPlugin';
    PLUGIN_TABCAPTION = 'Model Sync';
    PLUGIN_RIBBONGROUP_CONTROLNAME = 'ModelSync';
    PLUGIN_RIBBONGROUP_NAME = 'Model Sync';
  begin
    if Assigned(p_winRoadApplication) and not Assigned(p_Ribbon) then begin
      p_Ribbon:= p_winRoadApplication.mainForm.GetMainRibbonMenu;
      if not Assigned(p_RibbonTab) then begin
        p_RibbonTab:= p_Ribbon.CreateRibbonTab(PLUGIN_TABCONTROLNAME, 10000);
        p_RibbonTab.Caption:= PLUGIN_TABCAPTION;
      end;
      if not Assigned(p_RibbonGroup) then begin
        p_RibbonGroup:= p_RibbonTab.CreateRibbonGroup(PLUGIN_RIBBONGROUP_CONTROLNAME,7);
        P_RibbonGroup.Caption:= PLUGIN_RIBBONGROUP_NAME;
      end;
      if not Assigned(FormModelSyncResource) then begin
        FormModelSyncResource:= TFormModelSyncResource.Create(nil);
        FormModelSyncResource.ButtonModelSyncClient.Onclick:= ModelSyncClientClick;
        FormModelSyncResource.ButtonModelSyncServer.OnClick:= ModelSyncServerClick;
        FormModelSyncResource.ButtonModelCommandList.OnClick:= ModelCommandFormClick;
        FormModelSyncResource.Button3DTextList.OnClick:= ThreeDeeTextFormClick;
        FormModelSyncResource.Button3DTextList.Hide;
        if not Assigned(ModelCommandForm) then begin
          ModelCommandForm:= TModelCommandForm.Create(nil);
        end;
      end;
      if Assigned(p_RibbonGroup) then begin
        p_RibbonGroup.Width:= FormModelSyncResource.MainPanel.Width;
        p_RibbonGroup.AddGroupControl(FormModelSyncResource.MainPanel);
      end;
    end;
    p_Ribbon:= nil;
  end;

//=== finish ===================================================================
procedure TModelSyncPlugin.ModelSyncClientClick(Sender: TObject);
  begin
    if not Assigned(ModelSyncClientForm) then begin
      ModelSyncClientForm:= TModelSyncClientForm.Create(nil);
    end;
    ModelSyncClientForm.Show;
  end;

procedure TModelSyncPlugin.ModelSyncServerClick(Sender: TObject);
  begin
    if not Assigned(ModelSyncServerForm) then begin
      ModelSyncServerForm:= TModelSyncServerForm.Create(nil);
    end;
    ModelSyncServerForm.Show;
  end;

procedure TModelSyncPlugin.ModelCommandFormClick(Sender: TObject);
  begin
    if not Assigned(ModelCommandForm) then begin
      ModelCommandForm:= TModelCommandForm.Create(nil);
    end;
    ModelCommandForm.show;
  end;

procedure TModelSyncPlugin.ThreeDeeTextFormClick(Sender: TObject);
begin
  if not Assigned(ThreeDeeTextForm) then begin
    ThreeDeeTextForm:= TThreeDeeTextForm.Create(nil);
  end;
  ThreeDeeTextForm.show;
end;

procedure TModelSyncPlugin.ProjectDestroyProc;
  begin
    RemoveForms;
  end;

procedure TModelSyncPlugin.PaintScene(const opengl: TF8OpenGL);
  begin

  end;

procedure TModelSyncPlugin.PaintHUD(const opengl: TF8OpenGL);
  begin

  end;

procedure TModelSyncPlugin.SetupCamera(const opengl: TF8OpenGL);
  begin
    if Assigned(ModelCommandForm) then
      ModelCommandForm.SetupCamera;
  end;

procedure TModelSyncPlugin.RemoveForms;
  begin
    if Assigned(ModelSyncClientForm) then
      FreeAndNil(ModelSyncClientForm);
    if Assigned(ModelSyncServerForm) then
      FreeAndNil(ModelSyncServerForm);
    if Assigned(ModelCommandForm) then
      FreeAndNil(ModelCommandForm);
    if Assigned(ThreeDeeTextForm) then
      FreeAndNil(ThreeDeeTextForm);
    if Assigned(ChooseThreeDeeModel) then
      FreeAndNil(ChooseThreeDeeModel);
    if Assigned(ConfigRoadObstruction) then
      FreeAndNil(ConfigRoadObstruction);
    if Assigned(DeleteRoadObstruction) then
      FreeAndNil(DeleteRoadObstruction);
  end;

end.
