unit ModelSyncThread;

interface

uses
  System.Classes,
  ModelSyncUtil;

const
  BUFLEN = 100;

type

TOperationEvent = procedure(count: Integer)of object;

// ͻʹõḶ́߳Ҫշָ
TMSClientThread = class(TThread)
  private
    FSocket: TMSTCPClientSocket;
    FOnReceive: TOperationEvent;
    procedure CallOnReceive;
    procedure DoReceive(buf: Array of Byte);

  protected
    procedure Execute; override;

  public
    constructor Create(Socket: TMSTCPClientSocket; Suspended: Boolean);
    destructor Destroy;

    property OnReceive: TOperationEvent read FOnReceive write FOnReceive;
    property SocketHandle: TMSTCPClientSocket read FSocket;
  end;

TMSMovingSyncThread = class(TThread)
  private
    FSocket: TMSTCPClientSocket;
    procedure MovingInstanceSync;

  protected
    procedure Execute; override;

  public
    constructor Create(Socket: TMSTCPClientSocket; Suspended: Boolean);
    destructor Destroy;

    property SocketHandle: TMSTCPClientSocket read FSocket;
  end;

// ʹõḶ́߳ҪղָҷַеĿͻ
TMSServerThread = class(TThread)
  private
    FSocket: TMSTCPServerSocket;
    FOnReceive: TOperationEvent;
    FOperationCount: Integer;
    procedure CallOnReceive;

  protected
    procedure Execute; override;

  public
    constructor Create(ip,port:AnsiString; Suspended: Boolean);
    destructor Destroy;

    property OnReceive: TOperationEvent read FOnReceive write FOnReceive;
    property SocketHandle: TMSTCPServerSocket read FSocket;
  end;

implementation

uses
 WinSock,
 vcl.dialogs,
 SysUtils,
 MS_Operation,
 F8OpenGL,
 F8GLUtils,
 MS_Parser,
 PluginCore,
 ModelSyncPluginMain;

// ʼ
constructor TMSClientThread.Create(Socket: TMSTCPClientSocket; Suspended: Boolean);
  var
    Data: TWSAData;
  begin
    WSAStartup($0101, Data);
    if Assigned(Socket) then begin
      FSocket:= TMSTCPClientSocket.Create;
      FSocket.RemoteHost:= Socket.RemoteHost;
      FSocket.RemotePort:= Socket.RemotePort;
      FSocket.Open;
    end;
    FOnReceive:= nil;
    Inherited Create(Suspended);
  end;

constructor TMSMovingSyncThread.Create(Socket: TMSTCPClientSocket; Suspended: Boolean);
  var
    Data: TWSAData;
  begin
    WSAStartup($0101, Data);
    if Assigned(Socket) then begin
      FSocket:= TMSTCPClientSocket.Create;
      FSocket.RemoteHost:= Socket.RemoteHost;
      FSocket.RemotePort:= Socket.RemotePort;
      FSocket.Open;
    end;
    Inherited Create(Suspended);
  end;

constructor TMSServerThread.Create(ip: AnsiString; port: AnsiString; Suspended: Boolean);
  var
    Data: TWSAData;
  begin
    WSAStartup($0101, Data);
    FSocket:= TMSTCPServerSocket.Create;
    FSocket.LocalHost:= ip;
    FSocket.LocalPort:= port;
    FSocket.Open;
    FSocket.Bind;
    FSocket.Listen(10);
    FOnReceive:= nil;
    FOperationCount:= 0;
    Inherited Create(Suspended);
  end;

destructor TMSClientThread.Destroy;
  begin
    if Assigned(FSocket) then begin
      FreeAndNil(FSocket);
    end;
    FOnReceive:= nil;
    Inherited Destroy;
  end;

destructor TMSMovingSyncThread.Destroy;
  begin
    if Assigned(FSocket) then begin
      FreeAndNil(FSocket);
    end;
    Inherited Destroy;
  end;

destructor TMSServerThread.Destroy;
  begin
    if Assigned(FSocket) then begin
      FreeAndNil(FSocket);
    end;
    FOnReceive:= nil;
    FOperationCount:= 0;
    Inherited Destroy;
  end;

// FOnReceive
procedure TMSClientThread.CallOnReceive;
  begin
    if Assigned(FOnReceive) then begin
      FOnReceive(FSocket.OperationReceived);
    end;
  end;

procedure TMSServerThread.CallOnReceive;
  begin
    if Assigned(FOnReceive) then begin
      FOnReceive(FSocket.OperationReceived);
    end;
  end;

// ִ߳к
procedure TMSClientThread.Execute;
  var
    NewRecv: Integer;
    TimeInterval: TTimeVal;
    RecvSet: TFDSet;
    CurrentSet: TFDSet;

    RecvLen: Integer;
    RecvBuf: Array[0..BUFLEN - 1] of Byte;

  begin
    TimeInterval.tv_sec:= 0;
    TimeInterval.tv_usec:= 0;
    FillChar(RecvBuf, BUFLEN, 0);
    FD_ZERO(RecvSet);
    FD_SET(FSocket.Handle,RecvSet);

    while not Terminated do begin
      if FSocket.Connected then begin
        CurrentSet:= RecvSet;
        NewRecv:= select(0, @CurrentSet, nil, nil, @TimeInterval);
        if NewRecv > 0 then begin
          RecvLen:= FSocket.Receive(RecvBuf, BUFLEN, 0);
          if RecvLen > 0 then begin
            DoReceive(RecvBuf);
            Synchronize(CallOnReceive);
            fillchar(RecvBuf,BUFLEN,0);
            RecvLen:= 0;
            NewRecv:= 0;
          end;
        end;
      end;
    end;
  end;

procedure TMSClientThread.DoReceive(buf: array of Byte);
  var
    Parser: TMSOperationParser;
  begin
    Parser:= TMSOperationParser.Create;
    Parser.ToOperation(buf);
//    if Parser.CurrentOperation.Valid then
////      showmessage('Ч')
//    else
//      showmessage('Ч');
    Parser.CurrentOperation.Execute;
  end;

procedure TMSMovingSyncThread.Execute;
  begin
    while not Terminated do begin
      MovingInstanceSync;
      sleep(1000);
    end;
  end;

procedure TMSMovingSyncThread.MovingInstanceSync;
  var
    car: IF8CarInstance;
    project: IF8ProjectForRoad;
    i: Integer;
    index: Integer;
    requestOperation: TBaseOperation;
    parser: TMSOperationParser;
    sendBuf: Array [0..99] of Byte;
  begin
    car:= ModelSyncPlugin.winRoadApplication.mainForm.driver.currentCar;
    project:= ModelSyncPlugin.winRoadApplication.project;
    if Assigned(car) then begin
      parser:= TMSOperationParser.Create;
      for I := 1 to project.numberOfThreeDModels do begin
        if project.threeDModel[i].id = car.model.id then begin
          index:= i;
          break;
        end;
      end;
      requestOperation:= TSyncMovingInstanceOperation.Create(index, car.ID, car.instancePosition, car.yawAngle, car.rollAngle, car.pitchAngle, false);
      parser.ToByteArray(requestOperation);
      if FSocket.Connected then begin
        for I := 0 to 99 do
          sendBuf[i]:= parser.Buffer[i];
        FSocket.Send(sendBuf, 100, 0);
      end;
    end;
  end;

procedure TMSServerThread.Execute;
  var
    TimeInterval: TTimeVal;
    CurrentSet: TFDSet;
    ListenSockSet: TFDSet;
    ConnectSockSet: TFDSet;

    Addr: TSockAddr;
    AddrLen: Integer;
    ClientSock: TSocket;

    RecvLen: Integer;
    SendLen: Integer;
    RecvBuf: Array[0..BUFLEN - 1] of Byte;

    NewConnect: Integer;
    NewRecv: Integer;
    I: Integer;

    parser: TMSOperationParser;
  J: Integer;
  begin
    TimeInterval.tv_sec:= 0;
    TimeInterval.tv_usec:= 0;
    FD_ZERO(ConnectSockSet);
    FD_ZERO(ListenSockSet);
    FD_SET(FSocket.Handle,ListenSockSet);

    AddrLen:= sizeof(Addr);
    Fillchar(Addr, sizeof(Addr), 0);

    FillChar(RecvBuf, BUFLEN, 0);

    while FSocket.Listening and FSocket.Active and not Terminated do begin
      CurrentSet:= ListenSockSet;
      NewConnect:= select(0,@CurrentSet,nil,nil,@TimeInterval);
      if NewConnect > 0 then begin
        ClientSock:= accept(FSocket.Handle, @Addr, @AddrLen);
        if ClientSock <> INVALID_SOCKET then begin
          // ͻ׽ֶ
          FD_SET(ClientSock,ConnectSockSet);
          // ʼ
          ClientSock:= INVALID_SOCKET;
          AddrLen:= sizeof(Addr);
          Fillchar(Addr, sizeof(Addr), 0);
        end;
        NewConnect:= 0;
      end;

      // ÿͻ׽ֽвѯǷбĴ
      for I:= 0 to ConnectSockSet.fd_count - 1 do begin
        FD_ZERO(CurrentSet);
        FD_SET(ConnectSockSet.fd_array[I],CurrentSet);
        NewRecv:= select(0,@CurrentSet,nil,nil,@TimeInterval);
        if NewRecv > 0 then begin
          RecvLen:= WinSock.recv(ConnectSockSet.fd_array[I], RecvBuf, 100, 0);
          // ȡ˱ģԱĽд
          if RecvLen > 0 then begin
            parser:= TMSOperationParser.Create;
            if (RecvBuf[1] = MODEL_SYNC_CREATE_MODEL) or (RecvBuf[1] = MODEL_SYNC_CREATE_FIRE) then begin
              Inc(FOperationCount);
              Move(FOperationCount, RecvBuf[3], 4);
            end;
            parser.ToOperation(RecvBuf);
            parser.CurrentOperation.Validate;
            if parser.CurrentOperation.Valid then
              RecvBuf[2]:= 1
            else
              RecvBuf[2]:= 0;
            for J := 0 to ConnectSockSet.fd_count - 1 do begin
              if (J mod 3 = 2) then begin
                if RecvBuf[1] = MODEL_SYNC_SYNC_MOVING_INSTANCE then begin
                  if J <> (I + 1) then
                    SendLen:= WinSock.send(ConnectSockSet.fd_array[J], RecvBuf, 100, 0);
                end
                else
                  SendLen:= WinSock.send(ConnectSockSet.fd_array[J], RecvBuf, 100, 0);
              end;
            end;
            RecvLen:= 0;
            FillChar(RecvBuf, BUFLEN, 0);
          end;
          NewRecv:= 0;
        end;
      end;
    end;
  end;

end.
