unit ModelSyncUtil;

interface

uses
  SysUtils,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  WinSock;

//const
//  WSASTARTUPFAILED        = 2;
//  INVALIDSOCKET           = 4;
//  BINDFAILED              = 6;
//  INVALIDIPADDRESS        = 8;

type

TSocketProtocol = Word;
TSocketType = Integer;
TSocketDomain = Integer;

TSocketHost = type AnsiString;
TSocketPort = type AnsiString;

TMSIPSocket = class
  private
    FActive: Boolean;
    FSocket: TSocket;
    FProtocol: TSocketProtocol;
    FSockType: TSocketType;
    FDomain: TSocketDomain;
    FOperationReceived: Cardinal;
    FOperationSent: Cardinal;
    FLocalHost: TSocketHost;
    FLocalPort: TSocketPort;
    FRemoteHost: TSocketHost;
    FRemotePort: TSocketPort;

    procedure SetActive(Value: Boolean);
    procedure SetDomain(Value: TSocketDomain);
    procedure SetProtocol(Value: TSocketProtocol);
    procedure SetSockType(Value: TSocketType);

    procedure SetLocalHost(Value: TSocketHost);
    procedure SetLocalPort(Value: TSocketPort);
    procedure SetRemoteHost(Value: TSocketHost);
    procedure SetRemotePort(Value: TSocketPort);

  public
    constructor Create;
    destructor Destroy;
    function Bind: Boolean;
    procedure Open; virtual;
    procedure Close; virtual;
    function GetSocketAddr(h: TSocketHost; p: TSocketPort): TSockAddr;
    function LookupHostAddr(const hn: AnsiString): TSocketHost;
    function LookupPort(const pn: AnsiString): word;

    function Send(var buf; bufsize: Integer; flags: Integer): Integer;
    function Receive(var buf; bufsize: Integer; flags: Integer): Integer;

    property Active: Boolean read FActive write SetActive default False;
    property OperationReceived: Cardinal read FOperationReceived;
    property OperationSent: Cardinal read FOperationSent;
    property Domain: TSocketDomain read FDomain write SetDomain;
    property Handle: TSocket read FSocket;
    property Protocol: TSocketProtocol read FProtocol write SetProtocol;
    property SockType: TSocketType read FSockType write SetSockType;

    property LocalHost: TSocketHost read FLocalHost write SetLocalHost;
    property LocalPort: TSocketPort read FLocalPort write SetLocalPort;
    property RemoteHost: TSocketHost read FRemoteHost write SetRemoteHost;
    property RemotePort: TSocketPort read FRemotePort write SetRemotePort;
  end;

TMSTCPServerSocket = class(TMSIPSocket)
  private
    FListening: Boolean;
  public
    constructor Create;
    destructor Destroy;
    function Listen(backlog: Integer): Boolean;
    procedure Close; override;

    property Listening: Boolean read FListening;
  end;

TMSTCPClientSocket = class(TMSIPSocket)
  private
    FConnected: Boolean;
  public
    constructor Create;
    destructor Destroy;
    function Connect: Boolean;
    procedure Open; override;
    procedure Close; override;

    property Connected: Boolean read FConnected;
  end;

implementation

// ʼ
constructor TMSIPSocket.Create;
  begin
    FActive  := False;
    FOperationReceived := 0;
    FOperationSent := 0;
    FDomain := PF_INET;
    FProtocol := IPPROTO_IP;
    FSocket := INVALID_SOCKET;
    FSockType := SOCK_DGRAM;
    FLocalHost := '';
    FLocalPort := '';
    FRemoteHost := '';
    FRemotePort := '';
  end;

constructor TMSTCPServerSocket.Create;
  begin
    Inherited Create;
    Protocol:= IPPROTO_TCP;
    SockType:= SOCK_STREAM;
    FListening:= false;
  end;

constructor TMSTCPClientSocket.Create;
  begin
    Inherited Create;
    Protocol:= IPPROTO_TCP;
    SockType:= SOCK_STREAM;
    FConnected:= false;
  end;

// رsocket
// رմ͵ʾ
procedure TMSIPSocket.Close;
  begin
    if FActive then begin
      if CloseSocket(FSocket) <> 0 then begin
        showmessage('Socket Close Failed');
        Exit;
      end else
      begin
        FSocket:= INVALID_SOCKET;
        FActive:= false;
      end;
    end;
  end;

procedure TMSTCPServerSocket.Close;
  begin
    FListening:= false;
    Inherited Close;
  end;

procedure TMSTCPClientSocket.Close;
  begin
    if FConnected then begin
      if shutdown(FSocket, SD_BOTH) <> 0 then begin
        showmessage('Shutdown Failed');
      end else
      begin
        FConnected:= false;
      end;
    end;
    Inherited Close;
  end;

// պ
destructor TMSIPSocket.Destroy;
  begin
    Close;
    Inherited Destroy;
  end;

destructor TMSTCPServerSocket.Destroy;
  begin
    Inherited Destroy;
  end;

destructor TMSTCPClientSocket.Destroy;
  begin
    Inherited Destroy;
  end;

// һµsocket
procedure TMSIPSocket.Open;
  begin
    if not FActive then begin
      FSocket:= socket(FDomain, FSockType, FProtocol);
      FActive:= FSocket <> INVALID_SOCKET;
      if FActive then begin
        FOperationReceived:= 0;
        FOperationSent:= 0;
      end;
    end;
  end;

procedure TMSTCPClientSocket.Open;
  var
    addr: TSockAddr;
  begin
    Inherited Open;
    if Active and not FConnected then begin
      addr := GetSocketAddr(RemoteHost, RemotePort);
      FConnected := WinSock.connect(FSocket, addr, sizeof(addr)) = 0;
    end;
  end;

// ӵַ׽ӿ
function TMSIPSocket.Bind: Boolean;
  var
    addr: TSockAddr;
  begin
    Result := False;
    if FActive then begin
      addr:= GetSocketAddr(FLocalHost, FLocalPort);
      result:= WinSock.Bind(FSocket, addr, SizeOf(addr)) = 0;
    end;
  end;

// ipַͶ˿ַһַ
function TMSIPSocket.GetSocketAddr(h: TSocketHost; p: TSocketPort): TSockAddr;
  var
    lsHostAddr: AnsiString;
  begin
    Result.sin_family := AF_INET;
    lsHostAddr := AnsiString(LookupHostAddr(AnsiString(h)));
    Result.sin_addr.s_addr := inet_addr(PAnsiChar(lsHostAddr));
    Result.sin_port := htons(LookupPort(AnsiString(p)));
  end;

// 鿴ַЧ
function TMSIPSocket.LookupHostAddr(const hn: AnsiString): TSocketHost;
  begin
    if inet_addr(PAnsiChar(hn)) = -1 then begin
      showmessage('Invalid IP Address');
      result:= '0.0.0.0';
    end else
      result:= hn;
  end;

// 鿴˿ڵЧ
function TMSIPSocket.LookupPort(const pn: AnsiString): word;
  begin
    result:= StrTOInt(pn);
  end;

// ֽ
function TMSIPSocket.Send(var buf; bufsize: Integer; flags: Integer): Integer;
  begin
    Result:= WinSock.send(FSocket, buf, bufsize, flags);
    if Result <> SOCKET_ERROR then
      Inc(FOperationSent);
  end;

// ֽ
function TMSIPSocket.Receive(var buf; bufsize: Integer; flags: Integer): Integer;
  begin
    Result:= WinSock.recv(FSocket, buf, bufsize, flags);
    if Result <> SOCKET_ERROR then
      Inc(FOperationReceived);
  end;

// ʹOpenCloseȥActive״̬
procedure TMSIPSocket.SetActive(Value: Boolean);
  begin
    if FActive <> Value then begin
      if Value then Open
      else Close;
    end;
  end;

procedure TMSIPSocket.SetDomain(Value: Integer);
  begin
    if FDomain <> Value then
      FDomain:= Value;
  end;

procedure TMSIPSocket.SetProtocol(Value: Word);
  begin
    if FProtocol <> Value then
      FProtocol:= Value;
  end;

procedure TMSIPSocket.SetSockType(Value: Integer);
  begin
    if FSockType <> Value then
      FSockType:= Value;
  end;

procedure TMSIPSocket.SetLocalHost(Value: TSocketHost);
  begin
    if FLocalHost <> Value then
      FLocalHost:= Value;
  end;

procedure TMSIPSocket.SetLocalPort(Value: TSocketPort);
  begin
    if FLocalPort <> Value then
      FLocalPort:= Value
  end;

procedure TMSIPSocket.SetRemoteHost(Value: TSocketHost);
  begin
    if FRemoteHost <> Value then
      FRemoteHost:= Value;
  end;

procedure TMSIPSocket.SetRemotePort(Value: TSocketPort);
  begin
    if FRemotePort <> Value then
      FRemotePort:= Value;
  end;

function TMSTCPServerSocket.Listen(backlog: Integer): Boolean;
  begin
    if Active and not FListening then begin
      FListening:= WinSock.listen(Handle, backlog) = 0;
    end;
    result:= FListening;
  end;

function TMSTCPClientSocket.Connect: Boolean;
  begin
    Open;
    Result := FConnected;
  end;

end.
